/**
 * Copyright (c) 2008 Barcelona Supercomputing Center - Centro Nacional de
 * Supercomputacion, Heinrich-Heine-Universitaet Duesseldorf, and XLAB d.o.o.
 * All rights reserved.
 *
 * This file is part of DIXI. DIXI is part of XtreemOS, a Linux-based Grid
 * Operating System, see <http://www.xtreemos.eu> for more details. The
 * XtreemOS project has been developed with the financial support of the
 * European Commission's IST program under contract #FP6-033576.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the XtreemOS project nor the names of its
 *     partners and/or contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef XATICATYPES_H
#define XATICATYPES_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/**
 *  Enumerations for XATICA parameters, note that XOS_ prefix has been adopted
 *
 */
enum TypeOfInfo {
  XOS_BASIC=2,
  XOS_JOB_DEFINITION=4,
  XOS_RESOURCES_ALLOCATED=8,
  XOS_RESOURCES_CONSUMED=16,
  XOS_USER_METRICS=32,
  XOS_NO_BUFFER=64,
  XOS_XTRACE=128,
  XOS_ENABLE=256
};

enum InfoLevel {
  XOS_JOB=1,
  XOS_PROCESS,
  XOS_KERNEL
};

enum DependenceDirection {
  XOS_UP = 0,
  XOS_DOWN
};

enum ControlOperations {
  XOS_SUSPENDJOB=19,
  XOS_RESUMEJOB=18,
  XOS_CANCELJOB=15
};

enum MetricType {
  XOS_MTINT,
  XOS_MTDOUBLE,
  XOS_MTCHAR,
  XOS_MTSTRING,
  XOS_MTTIME
};

enum MetricScope {
  XOS_MSJOB,
  XOS_MSRESOURCE,
  XOS_MSJOBUNIT,
  XOS_MSPROCESS,
};

enum MetricBufSize {
  XOS_MBSHORT,
  XOS_MBMEDIUM,
  XOS_MBLONG,
};

enum JobHints {
	XOS_JHMIGRATION_REQ = 0,
	XOS_JHSCHEDULING_REQ = 1
};

enum SchedulingHints {
	XOS_SHDEFAULT=0,
	XOS_SHONEPERNODE=1
};

enum SharingValues {
	XOS_SVMUTUAL=0,
	XOS_SVEXCLUSIVE=1
};


/* XATICA parameters: ArrayList, CommunicationAddress and MetricsDesc */
struct arrayList
{
  char **list;
  int size;
};
typedef struct arrayList* ArrayList;

struct communicationAddress{
  struct in_addr host;
  struct in_addr *externalAddress;
  unsigned short int port;
};



typedef struct communicationAddress* CommunicationAddress;

struct gregorianCalendar{
 unsigned long long date;
};

typedef struct gregorianCalendar* GregorianCalendar;

/*
 * TTELM C struct, TODO: Fill.
 */
struct tTElm{
 unsigned long long startTime;
 unsigned long long endTime;
 int sharingValues;
 int currentAmount;
 char * uName;
 char * uValue;
};

typedef struct tTElm* TTElm;



//Modified jgonz 07/30/09
#define TTELM_DELIMITER_IN_LISTS "$TTE$"
#define TTELMREQ_DELIMITER_IN_LISTS "$TTR$"
#define RREQUEST_DELIMITER_IN_LISTS "$RR$"
#define UREQUEST_DELIMITER_IN_LISTS "$UR$"
#define RESERVATIONSLOT_DELIMITER_IN_LISTS "$RS$"

#define FREESLOTS_FIELDS_DELIMITER "#FS#"
#define TTELMREQ_FIELDS_DELIMITER "#TR#"
#define RREQUEST_FIELDS_DELIMITER "#RR#"
#define UREQUEST_FIELDS_DELIMITER "#UR#"
#define RESERVATIONSLOT_FIELDS_DELIMITER "#RS#"

//This value is not included in TTElmRequest structs anymore, though it's used when converting them into strings
#define RESID_FULLDESC_STRING "ReservationID"



struct linkedElement{
	void *element;
	struct linkedElement *next;
};


struct linkedList{
	//List head
	struct linkedElement *first;
	//Pointer to the last element
	struct linkedElement *last;
	//Number of members contained
	int size;
	//function used to free each of the contained elements
	void (* freeFunction)(void*);
	//Function used to calculate the equivalent string of each element
	int (* element2str)(void*,char**);
	//Function used to create a new element from a given string
	int (* str2element)(char*,void**);
	//Delimiter used to separate the elements when creating the equivalent string
	char *delim;
};

typedef struct linkedList* LinkedList;

struct reservationSlot{
	char *id;
	char *resourceID;
	CommunicationAddress location;
	LinkedList info;
};

typedef struct reservationSlot* ReservationSlot;

struct freeSlots{
	GregorianCalendar firstOccupied;
	GregorianCalendar lastOccupied;
	//List of TTElm's
	LinkedList free;
};
typedef struct freeSlots* FreeSlots;

/*
 * Equivalent to Request.java
 */
struct allocModificationRequest{
	//List of TTElmRequest's
	LinkedList elmReqs;
};
typedef struct allocModificationRequest* AllocModificationRequest;

struct reservationRequest{
	CommunicationAddress nodeAddress;
	AllocModificationRequest localRequest;
};
typedef struct reservationRequest* ReservationRequest;

struct updateRequest{
	CommunicationAddress nodeAddress;
	char *reservationID;
	AllocModificationRequest localRequest;
};

typedef struct updateRequest* UpdateRequest;

enum RequestTypes {
	XOS_RTECHO,
	XOS_RTADD,
	XOS_RTREMOVE,
	XOS_RTDELAY,
	XOS_RTREVERSE,
	XOS_RTPURGE
};
struct tTElmRequest{
	//const char *RESID_FULLDESC_STRING = "ReservationID";
	char *resourceID;
	TTElm ttelm;
	enum RequestTypes type;
};
typedef struct tTElmRequest* TTElmRequest;

//Code should be reorganized in the future

LinkedList newTTElmList();
LinkedList newTTElmRequestList();
LinkedList newReservationRequestList();
LinkedList newUpdateRequestList();
LinkedList newReservationSlotList();

ReservationSlot newReservationSlot();
void freeReservationSlot(ReservationSlot rs);
int str2ReservationSlot(const char *str,ReservationSlot *rs);

TTElmRequest newTTElmRequest(const char *rid,TTElm ttelm);
void freeTTElmRequest(TTElmRequest ttelmr);
int TTElmRequest2str(TTElmRequest ttelmr,char **str);

AllocModificationRequest newAllocModificationRequest();
void freeAllocModificationRequest(AllocModificationRequest amr);
int AMRequest2str(AllocModificationRequest req,char **str);

ReservationRequest newReservationRequest();
void freeReservationRequest(ReservationRequest  ttr);
int RRequest2str(ReservationRequest req,char **str);

UpdateRequest newUpdateRequest();
void freeUpdateRequest(UpdateRequest  ttr);
int URequest2str(UpdateRequest req,char **str);

FreeSlots newFreeSlots();
int str2FreeSlots(const char *str,FreeSlots *fs);
void freeFreeSlots(FreeSlots fs);

LinkedList newLinkedList();
int addElementToLinkedList(LinkedList list, void *element);
void freeLinkedList(LinkedList list);
int list2str(LinkedList list,char **strout);
int str2list(const char *str,LinkedList *list);



struct arrayListCA
{
  CommunicationAddress *list;
  int size;
};
typedef struct arrayListCA* ArrayListCA;

struct metricsDesc
{
  char *name;
  enum MetricType type;
  char *description;
  enum MetricScope scope;
  int bufferable;
  enum MetricBufSize bufsize;
};
typedef struct metricsDesc* MetricsDesc;

/**
 *  Helper structures to parse XML output
 *  These have constructors/destructors (new or XML2/free) and
 *  must be used through the specified functions in this file.
 *
 *  -=For example=-
 *
 *  JobInfoList myJil;
 *  ArrayList jobs;
 *
 *  myJil = XML2JobInfoList(jInfoXML);
 *  if (myJil == NULL){
 *    fprintf(stderr, "parse error\n");
 *    return -1;
 *  }
 *
 *  int errcode = getJobs(jil, &jobs);
 */
typedef struct
{
  unsigned long long timestamp;
  char *value;
} TimedValue;

typedef struct
{
  TimedValue **list;
  int size;
} ArrayListTV;

struct metricValue
{
  char *name;
  enum MetricType type;
  ArrayListTV values;
};
typedef struct metricValue* MetricValue;

typedef struct{
  void **list;
  int size;
  void **list2;
  int size2;
  CommunicationAddress resourceId;
  int pid;
} ArrayListXM;

typedef struct{
  char *jobId;
  ArrayListXM jMetrics;
  ArrayListXM rMetrics;
  ArrayListXM metrics;
}JobInfo;

struct jobInfoList
{
  JobInfo **list;
  int size;
};
typedef struct jobInfoList* JobInfoList;

struct arrayListInt
{
  int *list;
  int size;
};
typedef struct arrayListInt* ArrayListInt;

enum EventType {
  CHANGE_TO=1,
  GREATER_THAN,
  GREATER_OR_EQUAL_TO,
  EQUAL_TO,
  LESS_THAN,
  LESS_THAN_OR_EQUAL_TO,
  FULL_BUFFER
};

struct metricEvent{
  enum EventType type;
  char *metricName;
  char *value;
};
typedef struct metricEvent* MetricEvent;

/* ReservationRequest
struct reservationRequest
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct reservationRequest* ReservationRequest;

struct arrayListReservationRequest
{
	ReservationRequest *request;
	int size;
};
typedef struct arrayListReservationRequest* ArrayListReservationRequest;

 UpdateRequest
struct updateRequest
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct updateRequest* UpdateRequest;
struct arrayListUpdateRequest
{
	UpdateRequest *request;
	int size;
};
typedef struct arrayListUpdateRequest* ArrayListUpdateRequest;

 ReservationSlot
struct reservationSlot
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct reservationSlot* ReservationSlot;
struct arrayListReservationSlot
{
	ReservationSlot *request;
	int size;
};
typedef struct arrayListReservationSlot* ArrayListReservationSlot;
*/
/* XATICA parameters' functions */
/* ArrayList */
ArrayList newArrayList();
void freeArrayList(ArrayList array);
int addToArrayList(ArrayList array, char *el);
char *getFromArrayList(ArrayList array, int elno);
char *ArrayList2XML(ArrayList array);

/* CommunicationAddress */
int CommAddress2str(const CommunicationAddress ca, char **str);
int str2CommAddress(const char *str, CommunicationAddress *ca);
int equalsCommAddress(const CommunicationAddress ca1, const CommunicationAddress ca2);
void freeCommAddress(CommunicationAddress ca1);

/* MetricsDesc */
MetricsDesc newMetricsDesc();
MetricsDesc XML2MetricsDesc(const char *xmlin);
char *MetricsDesc2XML(const MetricsDesc md);
void setMetricsDescName(MetricsDesc md, const char *name);
void setMetricsDescDescription(MetricsDesc md, const char *description);
void setMetricsDescScope(MetricsDesc md, enum MetricScope ms);
void setMetricsDescType(MetricsDesc md, enum MetricType mt);
void setMetricsDescBufferable(MetricsDesc md, int bufferable);
void setMetricsDescBufSize(MetricsDesc md, enum MetricBufSize mb);
void freeMetricsDesc(MetricsDesc md);

/* GregorianCalendar */
int str2Long (const char * str, long * lo);


/* Helper structures' functions */
/* JobInfoList*/
/**
 * This function constructs a JobInfoList with the contents of a XML String
 * conforming to the JobInfoList Schema. The whole tree is stored in memory
 * so there is also a function to free it.
 * @param xmlin a string with the XML message received from getJobsInfo
 * @return NULL if XML doesn't validate.
 * @see #freeJobInfoList(JobInfoList jil)
 * FIXME add @deprecated offers no XSD validation
 */
JobInfoList XML2JobInfoList(const char *xmlin);

/**FIXME: Uncomment when xmlSchemaValidateDoc works.
 * This function constructs a JobInfoList with the contents of a XML String
 * conforming to the JobInfoList Schema. The whole tree is stored in memory
 * so there is also a function to free it.
 * @param xmlin a string with the XML message received from getJobsInfo
 * @return NULL if XML doesn't validate.
 * @see #freeJobInfoList(JobInfoList jil)
 */
//JobInfoList xml2JobInfoList(const char *xmlin, const char *schemasLocation);

void freeJobInfoList(JobInfoList jil);
int getJobs(const JobInfoList jil, ArrayList *array);
int getJobResources(const JobInfoList jil, const char *jobId, ArrayListCA *array);
int getResourcePids(const JobInfoList jil, const char *jobId, const CommunicationAddress ca, ArrayListInt *array);

/* MetricValue*/
/**
 * This function searches for a specific metric in the JobInfoList
 * structure by its name, scope, resourceId and pid when needed.
 * Then it sets the contents of a MetricValue structure with its
 * value/s (several when buffered).
 * @see #MetricValue
 * @param jil a pointer to the JobInfoList that contains the answer
 *            of getJobsInfo.
 * @see #XML2JobInfoList(const char *xmlin, JobInfoList *jil)
 * @param scope an enum MetricScope specifying the scope of the
 *        requested metric.
 * @param resourceId a CommunicationAddress specifying
 *        the resource, the requested metric belongs to. It can be
 *        NULL when not applicable: Job scope.
 * @param pid an integer specifying the process id of the process,
 *        the requested metric belongs to. It can be 0 when not
 *        applicable: Job/JobUnit/Resource scope.
 * @param metricName a string specifying the name of the metric.
 * @param mv a pointer to a MetricValue structure that will be set
 *        with the values from the metric.
 * @return error codes
 */
int getMetricValue(const JobInfoList jil, enum MetricScope scope, const char *jobId, const CommunicationAddress resourceId, const int pid, const char *metricName, MetricValue *mv);

void freeMetricValue(MetricValue mv);

/**
 * This function returns the first value present in the MetricValue
 * structure. It's the most common operation when using non-buffered
 * metrics.
 *
 * @param mv a pointer to the MetricValue struct.
 * @return a string with the first value of the MetricValue struct.
 */
char *getFirstMetricValue(const MetricValue mv);

char *getFromMetricValue(const MetricValue mv, int elno);
unsigned long long getTimestampFromMetricValue(const MetricValue mv, int elno);
int getMetricValueLength(const MetricValue mv);

/* ArrayListCA */
ArrayListCA newArrayListCA();
int addToArrayListCA(ArrayListCA array, CommunicationAddress el);
CommunicationAddress getFromArrayListCA(ArrayListCA array, int elno);
char *ArrayListCA2XML(ArrayListCA array);
void freeArrayListCA(ArrayListCA array);

/* ArrayListInt */
ArrayListInt newArrayListInt();
int addToArrayListInt(ArrayListInt array, int el);
int getFromArrayListInt(ArrayListInt array, int elno);
void freeArrayListInt(ArrayListInt array);

/* ReservationRequest
ReservationRequest *ReservationRequest2XML(char *xml);
char *XML2ReservationRequest(ReservationRequest request);
ArrayListReservationRequest XML2ArrayListReservationRequest(char *xml);
char *ArrayListReservationRequest2XML(ArrayListReservationRequest list);
*/
/* UpdateRequest
UpdateRequest *UpdateRequest2XML(char *xml);
char *XML2UpdateRequest(UpdateRequest request);
ArrayListUpdateRequest XML2ArrayListUpdateRequest(char *xml);
char *ArrayListUpdateRequest2XML(ArrayListUpdateRequest list);
*/
/* ReservationSlot
ReservationSlot *ReservationSlot2XML(char *xml);
char *XML2ReservationSlot(ReservationSlot slot);
ArrayListReservationSlot XML2ArrayListReservationSlot(char *xml);
char *ArrayListReservationSlot2XML(ArrayListReservationSlot list);
*/

/* GregorianCalendar */
int GregorianCalendar2str(const GregorianCalendar gc, char **str);
int str2GregorianCalendar(const char *str, GregorianCalendar *gc);
void freeGregorianCalendar(GregorianCalendar gc);


/* TTElm */
TTElm newTTElm(const unsigned long long startTimeMillis, const unsigned long long endTimeMillis, int sharing, const char * uName, const char * uValue, int currentAmount);
int TTElm2str(const TTElm tt, char **str);
int str2TTElm(const char *str, TTElm *tt);
void freeTTElm(TTElm tt);

#endif
 #ifdef __cplusplus
 }
 #endif
