#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * "global" method (i.e. gets invoked on the root address xosd)
	 */

int getDaemons( ArrayListCA* returnValue);


	/**
	 * Retrieve the list of names of the services currently running on this
	 * daemon.
	 * 
	 * @return The list of names of the currently running services.
	 */

int getServiceList( ArrayList* returnValue);


	/**
	 * Send to the root the list of services running on this node.
	 * 
	 * @return 0 if call was successful.
	 */

int registerMyServices( int* returnValue);


	/**
	 * "global" method (i.e. gets invoked on the root address xosd)
	 * 
	 * Retrieve a list of nodes that run a particular service.
	 * 
	 * @param serviceName
	 *            The name of the service being queried of.
	 * @return A list of addresses of the nodes that have registered the
	 *         service.
	 */

int getNodesRunningService(const char* __serviceName, ArrayListCA* returnValue);




int kill( int* returnValue);


	/**
	 * Attempts to start a service.
	 * @param serviceName The name of the service to start.
	 * @param numThreads The number of threads to create for the service. 1 is 
	 * usually sufficient.
	 * @param userCert The user credential. 
	 * @return True if the operation succeeded, and False otherwise.
	 * @throws Exception
	 */

int startService(const char* __serviceName,int __numThreads,const char* __userCert, char* returnValue);


	/**
	 * Attempts to stop a service.
	 * @param serviceName The name of the service to stop.
	 * @param userCert The user's credential.
	 * @return True if the operation succeeded. and False otherwise.
	 * @throws Exception
	 */

int stopService(const char* __serviceName,const char* __userCert, char* returnValue);


	/**
	 * Attempts to stop and then start a currently running service.
	 * @param serviceName The name of the service to restart.
	 * @param userCert The user's credential.
	 * @return True if the operation was successful, and False otherwise.
	 * @throws Exception
	 */

int restartService(const char* __serviceName,const char* __userCert, char* returnValue);


	/**
	 * Returns the information currently known on the given service.
	 * @param serviceName The service name to query for its status.
	 * @param userCert The user's credential.
	 * @return The status of the service.
	 * @throws Exception
	 */
/* TODO
 * Unimplemented return type: eu.xtreemos.system.eventmachine.stage.StageStatus
 */
int getServiceStatus(const char* __serviceName,const char* __userCert, void* returnValue);




int registerClient( );




int unregisterClient( );


#ifdef __cplusplus 
 } 
 #endif 
