#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
  /**
   * Enables sharing GPS. It is optional.
   * @param uuid the String of a Universally Unique IDentificator for the GPS
   * values to be published.
   */

int shareGPS(const char* __uuid,const char* __userCtx );


  /**
   * Disables sharing GPS. It is optional. If it is used all subscribers are removed.
   * @param uuid the String of the Universally Unique IDentificator for the GPS
   * @param userCtx xtreemos user certificate
   */

int unshareGPS(const char* __uuid,const char* __userCtx );


  /**
   * Update shared GPS position.
   * @param uuid the String of the Universally Unique IDentificator for the GPS
   * @param position Data published by GPS
   * @param userCtx xtreemos user certificate
   */

int updateGPSPosition(const char* __uuid,const char* __position,const char* __userCtx );


  /**
   * Return the number of subscribers for a mobile device shared GPS
   * @param uuid the String of the Universally Unique IDentificator for the GPS
   * @param userCtx xtreemos user certificate
   * @return The number of subscribers or zero
   */

int getSubscribers(const char* __uuid,const char* __userCtx, int* returnValue);


  /**
   * For subscribing jobs to shared GPS
   * @param uuid String of the Universally Unique IDentificator for the GPS
   * @param jobId String of the Universally Unique IDentificator for the job
   * @param userCtx xtreemos user certificate
   * @throws Exception When GPS is not shared
   */

int subscribeToGPS(const char* __uuid,const char* __jobId,const char* __userCtx );


  /**
   * For unsubscribing jobs to shared GPS
   * @param uuid String of the Universally Unique IDentificator for the GPS
   * @param jobId String of the Universally Unique IDentificator for the job
   * @param userCtx xtreemos user certificate
   * @throws Exception When GPS is not shared or job is not subscribed
   */

int unsubscribeToGPS(const char* __uuid,const char* __jobId,const char* __userCtx );


#ifdef __cplusplus 
 } 
 #endif 
