#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * Returns a list of resource descriptions describing the resources listed
	 * in the RCA DB as registered.
	 * 
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @return A collection of resource descriptors of resources registered with
	 *         the RCA.
	 */
/* TODO
 * Unimplemented return type: java.util.ArrayList<eu.xtreemos.xosd.resmng.data.ResourceDescriptorRecord>
 */
int getRegisteredResources(const char* __userCert, void* returnValue);


	/**
	 * Returns a list of resource descriptions describing the resources listed
	 * in the RCA DB as pending for registration.
	 * 
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @return A collection of resource descriptors of resources applied for
	 *         registration with the RCA.
	 */
/* TODO
 * Unimplemented return type: java.util.ArrayList<eu.xtreemos.xosd.resmng.data.ResourceDescriptorRecord>
 */
int getPendingResources(const char* __userCert, void* returnValue);



/* TODO
 * Unimplemented return type: eu.xtreemos.xosd.resmng.data.ResourceDescriptorRecord
 */
int getResourceRecord(const char* __id,const char* __userCert, void* returnValue);


	/**
	 * Returns a list of the IDs of the VOs that the RCA is a member of.
	 * 
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @return A collection of string representing the IDs of the VOs that this
	 *         RCA is a member of.
	 */

int getVOList(const char* __userCert, ArrayList* returnValue);


	/**
	 * Put the resource on the list of resources that can be registered, but
	 * need to wait for an authorised administrator to confirm the registration
	 * using the {@link #confirmRegistration confirmRegistration} call before
	 * the resource can have its certificates signed by the RCA.
	 * 
	 * If the resource is already on any of the lists, then their entry gets
	 * replaced with the new value, thus updating the application or the
	 * registration.
	 * 
	 * @param resource
	 *            The descriptor of the resource applying for the registration.
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @return 0 if the call was successful.
	 */

int applyForRegistration(const char* __resource,const char* __userCert, int* returnValue);


	/**
	 * For an already registered resource, update the resource description in
	 * the RCA DB.
	 * 
	 * @param resource
	 *            The descriptor of the resource with the updated data. The
	 *            descriptor should contain the proper resource's ID.
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @return
	 * @throws PermissionDeniedException
	 * @throws ResourceNotRegisteredException
	 */

int updateResource(const char* __resource,const char* __userCert, int* returnValue);


	/**
	 * Confirm the registration of a resource that has previously been applied
	 * for the registration using {@link #applyForRegistration
	 * applyForRegistration}. After this call, the RCA will sign certificates
	 * for the registered resource ({@link #requestCertificate
	 * requestCertificate}).
	 * 
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @param id
	 *            The id of the resource record signifying the resource to be
	 *            confirmed for the registration.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * 
	 * @return 0 if the call was successful.
	 */

int confirmRegistration(const char* __id,const char* __userCert, int* returnValue);



/* TODO
 * Unimplemented return type: eu.xtreemos.xosd.resmng.data.ResourceDescriptorRecord
 */
int unregisterResource(const char* __id,const char* __userCert, void* returnValue);


	/**
	 * Retrieves the current status of the resource according to the RCA DB.
	 * 
	 * @param id
	 *            The identifier of the resource to check the status of.
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @return The status of the resource.
	 */

int getResourceStatus(const char* __id,const char* __userCert, int* returnValue);




int notifyVOMembershipChange(const char* __id,const char* __vo,char __addition,const char* __userCert, int* returnValue);


	/**
	 * Manipulates the membership of this RCA in the VO.
	 * 
	 * By adding the RCA into a VO, all the registered resources are also
	 * notified about being added into the VO. Similarly, when the RCA is
	 * removed from a VO, the registered resources are notified as well.
	 * 
	 * @param vo
	 *            The ID of the VO that is the object of the call.
	 * @param setMember
	 *            True for adding the RCA into the VO, or false otherwise.
	 * @param userCert
	 *            The user's credential, used for limiting the access control to
	 *            the administrators. Clients such as the web interface server
	 *            code should use their appropriate server certificates.
	 * @return
	 * @throws PermissionDeniedException
	 *             Thrown if the userCert does not represent an authorised user.
	 * @throws Exception
	 */

int setVOMembership(const char* __vo,char __setMember,const char* __userCert, int* returnValue);



/* TODO
 * Unimplemented return type: java.util.Hashtable<java.lang.String, java.security.cert.X509Certificate>
 */
int fetchPublicCertificates( void* returnValue);



/* TODO
 * Unimplemented return type: java.util.Hashtable<eu.xtreemos.xosd.security.rca.ResourceID, java.security.cert.X509Certificate>
 */
int getMachineIdentityCertificates(const char* __resources, void* returnValue);



/* TODO
 * Unimplemented return type: java.util.Hashtable<eu.xtreemos.xosd.security.rca.ResourceID, java.security.cert.X509Certificate>
 */
int getMachineAttributeCertificates(const char* __resources, void* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
