#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * Queries the local monitor data provider for machine status and returns
	 * the values of the metrics being monitored on this node. The metrics
	 * returned are those listed by {@link #getResMetrics()} and regulated by
	 * {@link #addResAttribute(String)}.
	 * 
	 * If the monitorType entry in the configuration file is set to "ganglia",
	 * then the method obtains the information from the localhost ganglia
	 * daemon. If there is no monitoring system, the service will return a
	 * generic answer, contained in the file files/genericAnswer.xml .
	 * 
	 * Alternatively, if the configuration is set to monitorType = "xmonitor",
	 * then the information is obtained from local kernel hooks which need to be
	 * pre-installed.
	 * 
	 * The call is self-sustainable and does not involve other services.
	 * 
	 * @return A hashtable with metrics and their respective values, describing
	 *         the details of this node.
	 * @author eva.milosev@xlab.si
	 * 
	 *         If there is no monitoring system, the service will return a
	 *         generic answer, contained in the file files/genericAnswer.xml
	 */
/* TODO
 * Unimplemented return type: java.util.Hashtable<java.lang.String, java.lang.String>
 */
int queryResInfo( void* returnValue);


	/**
	 * Queries the local monitor data provider for machine status, translates it
	 * into GLUE 1.2-compliant XML and transforms it into a Hashtable.
	 * 
	 * If the monitorType entry in the configuration file is set to "ganglia",
	 * then the method obtains the information from the localhost ganglia
	 * daemon. If there is no monitoring system, the service will return a
	 * generic answer, contained in the file files/genericAnswer.xml .
	 * 
	 * Alternatively, if the configuration is set to monitorType = "xmonitor",
	 * then the information is obtained from local kernel hooks which need to be
	 * pre-installed.
	 * 
	 * The call is self-sustainable and does not involve other services.
	 * 
	 * @return Hashtable with the node's hardware and load information, in a
	 *         GLUE 1.2-compliant XML structure.
	 * @author eva.milosev@xlab.si
	 * 
	 */
/* TODO
 * Unimplemented return type: java.util.Hashtable<java.lang.String, java.lang.Object>
 */
int query( void* returnValue);


	/**
	 * Queries the local monitor data provider for machine status and returns a
	 * filled-in collection (that should at this point contain a single element)
	 * of resource descriptor records.
	 * 
	 * If the monitorType entry in the configuration file is set to "ganglia",
	 * then the method obtains the information from the localhost ganglia
	 * daemon. If there is no monitoring system, the service will return a
	 * generic answer, contained in the file files/genericAnswer.xml .
	 * 
	 * Alternatively, if the configuration is set to monitorType = "xmonitor",
	 * then the information is obtained from local kernel hooks which need to be
	 * pre-installed.
	 * 
	 * The call is self-sustainable and does not involve other services.
	 * 
	 * @return A collection of resource descriptors describing the node. At this
	 *         point the collection should contain a single element.
	 *         If there is no monitoring system, the service will return a
	 *         generic answer, contained in the file files/genericAnswer.xml
	 * @author eva.milosev@xlab.si
	 * 
	 */
/* TODO
 * Unimplemented return type: java.util.ArrayList<eu.xtreemos.xosd.resmng.data.ResourceDescriptorRecord>
 */
int queryResourceDescriptor( void* returnValue);




int getResMetrics( ArrayList* returnValue);




int addResAttribute(const char* __attribute, char* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
