#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * Returns a filter policy for the resource related with the user.
	 * @param xosUserCert The user certificate.
	 * @param resourceCert The attribute certificate describing the resource.
	 * @param jsdlContent The job description 
	 * @return Policy filter consisting of the rules for the resource and the 
	 * user.
	 * @throws Exception
	 */

int obtainResourceFilterPolicy(const char* __xosUserCert,const char* __resourceCert,const char* __jsdlContent, char** returnValue);




int obtainFilterPolicyAEM(const char* __xosUserCert,const char* __jsdlContent,const char* __policyType, char** returnValue);


	/**
	 * Used by AEM framework to check if resources listed in 
	 * {@list ResourceMatching} comply with policies stored in VO policy storage
	 * {@link PolicyFactory#listPolicies()}.  
	 * 
	 * @param xos_cert instance of {@link X509Certificate}.
	 * @param resources list of potential compliant resource, see also 
	 * ResourceMathching class.
	 * @param action String used to denote the action taken by the user over 
	 * resources. Use constants declared inside {@link XACMLConstants#Action}. 
	 * @return instance of ResourceMatching, also includes digital signature, 
	 * see {@link ResourceMatching#setSignature(byte[])}and
	 * {@link ResourceMatching#getSignature()}.
	 */
/* TODO
 * Unimplemented return type: eu.xtreemos.xosd.resmng.data.ResourceMatching
 */
int verifyPolicyAEM(const char* __xos_cert,const char* __resources,const char* __action, void* returnValue);


	/**
	 * Create Access request towards VOPS PDP. Direct call to VOPS database.
	 * 
	 * @param jsdl Document describing job
	 * @param userCtx user certificate
	 * @param resourceCtx resource certificate
	 * @param action one of {@link eu.xtreemos.security.vops.xacml.utils.common.XACMLConstants.ActionConstants#getAttributeName()}
	 * @return Integer - decision. One of {@link Result#DECISION_PERMIT}, {@link Result#DECISION_DENY}.
	 * @throws Exception
	 */

int accessRequest(const char* __jsdl,const char* __userCtx,const char* __resourceCtx,const char* __action, int* returnValue);


	/**
	 * This method is called by consequence of the 
	 * {@link VOPS#verifyPolicyAEM(Object, ResourceMatching, String)} method.
	 * Enforces policies - generates XACML request for each of the resource
	 * and checks it against policies residing in policy storage, see 
	 * {@link PolicyFactory#listPolicies()}.
	 *  
	 * @param xos_cert User certificate which has been stored in VOPSStorage
	 * @param resources are collection from process of resource mathcing, see 
	 * {@link ResourceMatching}. 
	 * @return instance of the ResourceMatching object. It contains a list of 
	 * all potential resource nodes.
	 */
/* TODO
 * Unimplemented return type: eu.xtreemos.xosd.resmng.data.ResourceMatching
 */
int verifyPolicyCertRes(const char* __xos_cert,const char* __resources, void* returnValue);


	/**
	 * Policy with policyId will be removed from policy storage.
	 *
	 * @param policyId 
	 * @param complementaryId vo, user or resource id.
 	 * @param policyType is one of {@link Conf#USER_STR}, {@link Conf#RESOURCE_STR} or
	 * {@link Conf#VO_STR}.
	 * @param processMonitoringRules if set to true, monitoring rules are extracted from
	 * the policy and events are created in monitoring facility.
	 * @param userCtx certificate of the subject executing this call.
	 * 
	 * @return true on success
	 * @throws instance of Exception, error from {@link PolicyFactory#removePolicy(String)}.
	 */

int removePolicy(const char* __policyId,const char* __complementaryId,const char* __policyType,char __processMonitoringRules,const char* __userCtx, char* returnValue);


	/**
	 * Note that list of all policies can be very large. See also 
	 * {@link VOPS#listFilteredPolicy(String)} 
	 * and {@link VOPS#listPolicy(String)}.
	 * 
	 * @param complementaryId vo, user or resource id.
	 * @param policyType is one of {@link Conf#USER_STR}, {@link Conf#RESOURCE_STR} or
	 * {@link Conf#VO_STR}. 
	 * @param userCtx certificate of the subject executing this call.
	 * 
	 * @return ArrayList of XACML policies as ArrayList of strings.
	 * 
	 * @throws Exception policyType could not be one among those defined in {@link Conf}. 
	 */

int listPolicies(const char* __complementaryId,const char* __policyType,const char* __userCtx, ArrayList* returnValue);


	/**
	 * Lists specific policy with policyId. Returns XACML policy as String object.
	 * 
	 * @param policy id.
	 * @param complementaryId
	 * @param policyType is one of {@link Conf#USER_STR}, {@link Conf#RESOURCE_STR} or
	 * {@link Conf#VO_STR}.
	 * @param userCtx certificate of the subject executing this call.
	 * 
	 * @return XACML policy as a string.
	 * 
	 * @throws Exception policyType could not be one among those defined in {@link Conf}. 
	 */

int listPolicy(const char* __policyId,const char* __complementaryId,const char* __policyType,const char* __userCtx, char** returnValue);


	/**
	 * Adds XACML policy into policy storage {@link EXistsDb}. If adding a resource
	 * policy, additional check is made by checking the resource id in resCert 
	 * and xacmlPolicy. 
	 *
	 * @param Policy in XML passed as string (XACML format).
	 * @param policyId is the id of the policy in the database. If null, id will 
	 * be generic id will be used.
	 * @param complementaryId user, VO or resource Id, depends on the type of policy
	 * @param policyType is one of {@link Conf#USER_STR}, {@link Conf#RESOURCE_STR} or
	 * {@link Conf#VO_STR}.
	 * @param processMonitoringRules if set to true, monitoring rules are extracted from
	 * the policy and events are created in monitoring facility. 
	 * @param userCtx certificate of the subject executing this call.	 
	 *  
	 * @return policyId which has been added or modified. 
	 * @throws Exception policyType could not be one among those defined in {@link Conf}.
	 */

int addPolicy(const char* __xacmlPolicy,const char* __policyId,const char* __complementaryId,const char* __policyType,char __processMonitoringRules,const char* __userCtx, char** returnValue);


	/**
	 * Writes back policies from policy storage in PolicyFactory on to local disk. 
	 * 
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success.
	 */

int writeBack(const char* __userCtx, char* returnValue);


	/**
	 * Reloads all policies stored in policy storage. Only VO admin is capable of
	 * exec this command.
	 * 
	 * @param policyType the type of the user.
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success
	 */

int reloadVopsPDP(const char* __policyType,const char* __userCtx, char* returnValue);


	/**
	 * Reloads all policies stored in policy storage.
	 * 
	 * @param policyType the type of user
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success
	 */

int reloadDatabase(const char* __policyType,const char* __userCtx, char* returnValue);


	/**
	 * Returns a list of monitoring rules known by VOPS. These rules have been
	 * added by issuing command {@link VOPS#addPolicy(String, String, String, String, Boolean, X509Certificate)}
	 * and setting attribute processMonitoringRules to true.
	 * 
	 * @return an ArrayList containing monitoring rules. 
	 */

int getMonitoringRules( ArrayList* returnValue);


	/**
	 * Properly shuts down the eXists database.
	 * 
	 * @param policyType type of the user
	 * @param userCtx 
	 */

int properDBShutDown(const char* __policyType,const char* __userCtx, char* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
