/**
 * Copyright (c) 2008 Barcelona Supercomputing Center - Centro Nacional de
 * Supercomputación, Heinrich-Heine-Universität Düsseldorf, and XLAB d.o.o.
 * All rights reserved.
 *
 * This file is part of DIXI. DIXI is part of XtreemOS, a Linux-based Grid
 * Operating System, see <http://www.xtreemos.eu> for more details. The
 * XtreemOS project has been developed with the financial support of the
 * European Commission's IST program under contract #FP6-033576.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the XtreemOS project nor the names of its
 *     partners and/or contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author jgiralt @ bsc
 * @see xos_errlist.h
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef XOS_ERRNO_H
#define XOS_ERRNO_H

  /* System errors */
#define _xos_syserr_end 1000
#define _xos_syserr_start 900
#define XOS_ECOMMURCV 999
#define XOS_EPARSERCV 998
#define XOS_ECOMMUSND 997
#define XOS_EPARSESND 996
#define XOS_EMSGBUS	  995  /* Message bus error */

  /* System exceptions */
#define XOS_EXGENERIC     1   /* Generic exception */
#define XOS_EXBADRES      2   /* Bad resource */
#define XOS_EXBADATTRCERT 3   /* Bad attribute certificate of the resource */
#define XOS_EXRESNOTREG   4   /* Resource not registered */
#define XOS_EXUNKNOWNDT   5   /* Unknown datatype */
#define XOS_EXUNKNOWNOID  6   /* Unknown OID */
#define XOS_EXBADUSERCERT 7   /* Bad user certificate*/
#define XOS_EXREVDIR      8   /* Reservation directory generic exception */
#define XOS_EXREVDIRALRE  9   /* Reservation directory: job already attached */
#define XOS_EXREVDIRNOEMP 10  /* Reservation directory: not empty reservation */
#define XOS_EXREV         11  /* Reservation generic exception */
#define XOS_EXREVNORES    12  /* Reservation: not enough resources */
#define XOS_EXREVNOSLOT   13  /* Reservation: no free slots available */
#define XOS_EXREVLOCALLOC 14  /* Reservation: local allocation error */
#define XOS_EXREVRESELECT 15  /* Reservation: resource selection error */
#define XOS_EXREVACCDENY  16  /* Reservation: access denied */
#define XOS_EXREVNOEXIST  17  /* Reservation: does not exist */
#define XOS_EXJOBNOEXIST  18  /* JobID doesn't exist */
#define XOS_EXINVCERT     19  /* Invalid certificate */
#define XOS_EXOPNOALLOW   20  /* Operation not allowed */
#define XOS_EXNOACCES     21  /* Permission denied */
#define XOS_EXBADMETRIC   22  /* Bad metric */
#define XOS_EXMON         23  /* Monitor generic exception */
#define XOS_EXREVFUTURE   24  /* Reservation starts in the future */
#define XOS_EXREVEXPIRE   25  /* Reservation expired */
#define XOS_EXNORESAV     26  /* No resource available */
#define XOS_EXNORESPOL    27  /* No resource matches policies */
#define XOS_EXRESNOIREV   28  /* Resource not in reservation */
#define XOS_EXBADSIG      29  /* Bad signature */
#define XOS_EXSVCNORUN    30  /* Service not running */
#define XOS_JSDLPARSEERR  31  /* Badly formed JSDL */
#define XOS_EXVOPSFILTERR 32  /* Error obtaining the policy filter */

#endif
#ifdef __cplusplus
}
#endif
